
/***********************************************************************
* Copyright 1998-2020 CERN for the benefit of the EvtGen authors       *
*                                                                      *
* This file is part of EvtGen.                                         *
*                                                                      *
* EvtGen is free software: you can redistribute it and/or modify       *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation, either version 3 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* EvtGen is distributed in the hope that it will be useful,            *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with EvtGen.  If not, see <https://www.gnu.org/licenses/>.     *
***********************************************************************/

#ifndef EVTSTRINGPARTICLE_HH
#define EVTSTRINGPARTICLE_HH

#include "EvtGenBase/EvtId.hh"
#include "EvtGenBase/EvtParticle.hh"

// Description: EvtStringParticle particle is a class to keep the
//              collection of partons that are generated by jetset.
//              This is supposedly the _only_ use of this particle.

class EvtStringParticle : public EvtParticle {
  public:
    void init( EvtId id, const EvtVector4R& p4 ) override;
    void initPartons( int npartons, EvtVector4R* p4partons, EvtId* idpartons );
    int getNPartons();
    EvtId getIdParton( int i );
    EvtVector4R getP4Parton( int i );
    EvtSpinDensity rotateToHelicityBasis() const override;
    EvtSpinDensity rotateToHelicityBasis( double alpha, double beta,
                                          double gamma ) const override;

  private:
    std::vector<EvtVector4R> _p4partons;
    std::vector<EvtId> _idpartons;

    EvtStringParticle& operator=( const EvtStringParticle& d );
};
#endif
